@@ -67,6 +67,8 @@
 #' @export
 #'
 #' @examples
+#' if(interactive()){
+#'
 #' # ordinary use
 #' p <- with(iris, l_plot(Sepal.Width, Petal.Length, color=Species))
 #'
@@ -88,7 +90,7 @@
 #'             showScales = TRUE)
 #' plot(p)
 #'
-#'
+#' }
 l_plot <- function(x, y, ...) {
     UseMethod("l_plot")
 }
@@ -109,8 +111,36 @@ l_plot <- function(x, y, ...) {
 #'  See the function \code{\link{xy.coords}} for details.
 #'  If supplied separately, they must be of the same length.
 #' @param y argument description is as for the \code{x} argument above.
+#' @param by loon plot can be separated by some variables into multiple panels.
+#' This argument can take a \code{vector}, a \code{list} of same lengths or a \code{data.frame} as input.
+#' @param layout layout facets as \code{'grid'}, \code{'wrap'} or \code{'separate'}
+#' @param connectedScales Determines how the scales of the facets are to be connected depending
+#' on which \code{layout} is used.  For each value of \code{layout}, the scales are connected
+#' as follows:
+#' \itemize{
+#' \item{\code{layout = "wrap":}  Across all facets, when \code{connectedScales} is
+#'    \itemize{
+#'    \item{\code{"x"}, then  only the "x"  scales are connected}
+#'    \item{\code{"y"}, then only the "y" scales are connected}
+#'    \item{\code{"both"},  both "x" and "y" scales are connected}
+#'    \item{\code{"none"},  neither "x" nor "y" scales are connected.}
+#'    For any other value, only the "y" scale is connected.
+#'    }
+#'    }
+#' \item{\code{layout = "grid":}  Across all facets, when \code{connectedScales} is
+#'    \itemize{
+#'    \item{\code{"cross"}, then only the scales in the same row and the same column are connected}
+#'    \item{\code{"row"}, then both "x" and "y" scales of facets in the same row are connected}
+#'    \item{\code{"column"}, then both "x" and "y" scales of facets in the same column are connected}
+#'    \item{\code{"x"}, then all of the "x"  scales are connected (regardless of column)}
+#'    \item{\code{"y"}, then all of the "y" scales are connected (regardless of row)}
+#'    \item{\code{"both"},  both "x" and "y" scales are connected in all facets}
+#'    \item{\code{"none"},  neither "x" nor "y" scales are connected in any facets.}
+#'    }
+#'    }
+#'  }
 #' @param color colours of points (default "grey60"); colours are repeated
-#'  until matching the number points,
+#'  until matching the number points.
 #' @param glyph shape of point; must be one of the primitive glyphs
 #' "circle", "ccircle", "ocircle", "square", "csquare", "osquare", "triangle", "ctriangle",
 #' "otriangle", "diamond", "cdiamond", or "odiamond". Prefixes "c" and "o" mean closed and open, respectively.
@@ -120,13 +150,13 @@ l_plot <- function(x, y, ...) {
 #' serial axes glyphs may be added, but only after the plot has been created.
 #' @param size size of the symbol (roughly in terms of area)
 #' @param active a logical determining whether points appear or not
-#' (default is TRUE for all points). If a logical vector is given of length
-#' equal to the number of points, then it identifies which points appear (TRUE)
-#' and which do not (FALSE).
+#' (default is \code{TRUE} for all points). If a logical vector is given of length
+#' equal to the number of points, then it identifies which points appear (\code{TRUE})
+#' and which do not (\code{FALSE}).
 #' @param selected a logical determining whether points appear selected at first
-#' (default is FALSE for all points). If a logical vector is given of length
-#' equal to the number of points, then it identifies which points are (TRUE)
-#' and which are not (FALSE).
+#' (default is \code{FALSE} for all points). If a logical vector is given of length
+#' equal to the number of points, then it identifies which points are (\code{TRUE})
+#' and which are not (\code{FALSE}).
 #' @param xlabel Label for the horizontal (x) axis. If missing,
 #' one will be inferred from \code{x} if possible.
 #' @param ylabel Label for the vertical (y) axis. If missing,
@@ -147,21 +177,33 @@ l_plot <- function(x, y, ...) {
 #'   specified (i.e. not \code{NULL}) then the plot widget needs to be placed using
 #'   some geometry manager like \code{\link{tkpack}} or \code{\link{tkplace}} in
 #'   order to be displayed. See the examples below.
-#' @param ... named arguments to modify plot states.
+#' @param ... named arguments to modify plot states or layouts, see details.
 #'
 #'
-#' @details The scatterplot displays a number of direct interactions with the
+#' @details \itemize{
+#'   \item {The scatterplot displays a number of direct interactions with the
 #'   mouse and keyboard, these include: zooming towards the mouse cursor using
 #'   the mouse wheel, panning by right-click dragging and various selection
 #'   methods using the left mouse button such as sweeping, brushing and
 #'   individual point selection. See the documentation for \code{\link{l_plot}}
 #'   for more details about the interaction gestures.
+#'   }
+#'   \item {Some arguments to modify layouts can be passed through,
+#'   e.g. "separate", "byrow", etc. Check \code{\link{l_facet}}
+#'   to see how these arguments work.
+#'   }
+#' }
+#'
+#'
+#'
 #'
 #' @seealso  \code{\link{l_plot_arguments}}
 #' @export
 #' @export l_plot.default
 #'
 #' @examples
+#' if(interactive()){
+#'
 #'
 #' # default use as scatterplot
 #'
@@ -189,6 +231,8 @@ l_plot <- function(x, y, ...) {
 #'
 #' gridExtra::grid.arrange(loonGrob(p1), loonGrob(p2), nrow = 1)
 #'
+#' # Layout facets
+#' p <- with(mtcars, l_plot(wt, mpg, by = cyl, layout = "wrap"))
 #'
 #' # Use with other tk widgets
 #' tt <- tktoplevel()
@@ -205,7 +249,11 @@ l_plot <- function(x, y, ...) {
 #'
 #' tkgrid.rowconfigure(tt, 0, weight=1)
 #'
+#'}
 l_plot.default <-  function(x, y = NULL,
+                            by = NULL,
+                            layout = c("grid", "wrap", "separate"),
+                            connectedScales = c("cross", "row", "column", "both", "x", "y", "none"),
                             color = "grey60",
                             glyph = "ccircle",
                             size = 4,
@@ -221,108 +269,252 @@ l_plot.default <-  function(x, y = NULL,
                             background = "white",
                             parent = NULL, ...) {
 
-    ## Check for missing arguments
-    ## Get x, y, xlab and ylab
-    if (missing(xlabel)){
-        if (!missing(x)){
-            xlabel <- gsub("\"", "", deparse(substitute(x)))
-        } else {
-            xlabel <- ""
-        }
-    }
-
-    if (missing(ylabel)) {
-        if (!missing(y)){
-            ylabel <- gsub("\"", "", deparse(substitute(y)))
-        } else {
-            ylabel <- ""
-        }
-    }
-    if (missing(title)) { title <- "" }
+    args <- list(...)
+    # set by args, used for facetting
+    by_args <- args[byArgs]
+    # args passed into loonPlotFactory
+    args[byArgs] <- NULL
 
+    if(missing(title)) { title <- "" }
 
     if(missing(x)) {
 
-        plot <- loonPlotFactory('::loon::plot', 'plot',
-                                'loon scatterplot',
-                                parent,
-                                # No info about points
-                                # to be passed on
+        ## Check for missing arguments
+        ## Get x, y, xlab and ylab
+        if (missing(xlabel)){
+            xlabel <- ""
+        }
+
+        if (missing(ylabel)) {
+            ylabel <- ""
+        }
+        # Should we check `by` var here?
+        plot <- do.call(
+            loonPlotFactory,
+            c(
+                args,
+                list(
+                    factory_tclcmd = '::loon::plot',
+                    factory_path = 'plot',
+                    factory_window_title = 'loon scatterplot',
+                    parent = parent,
+                    # No info about points
+                    # to be passed on
+                    xlabel = xlabel,
+                    ylabel = ylabel,
+                    title = title,
+                    showLabels = showLabels,
+                    showScales = showScales,
+                    showGuides = showGuides,
+                    guidelines = guidelines,
+                    guidesBackground = guidesBackground,
+                    foreground = foreground,
+                    background = background
+                )
+            )
+        )
+
+        class(plot) <- c("l_plot", class(plot))
+        return(plot)
+
+    } else {
+
+        sync <- args$sync
+
+        if(is.null(sync)) {
+            sync <- "pull"
+            if(length(color) > 1) {
+                sync <- "push"
+            } else {
+                if(length(color) == 1 && !is.na(color) && color != "grey60") sync <- "push"
+            }
+
+            if(length(size) != 1) {
+                sync <- "push"
+            } else {
+                if(length(size) == 1 && !is.na(size) && size != 4) sync <- "push"
+            }
+
+            if(length(glyph) != 1) {
+                sync <- "push"
+            } else {
+                if(length(glyph) == 1 && !is.na(glyph) && glyph != "ccircle") sync <- "push"
+            }
+        }
+
+        ## Get x, y, xlab, ylab
+        ## similar to plot.default use of xy.coords
+        xlab <- deparse(substitute(x))
+        ylab <- deparse(substitute(y))
+        xOrigin <- x
+        xy <- xy.coords(x, y)
+        x <- xy$x
+        y <- xy$y
+
+        if (missing(xlabel)){
+            xlabel <- if (is.null(xy$xlab)) xlab else xy$xlab
+        }
+
+        if (missing(ylabel)) {
+            ylabel <- if (is.null(xy$ylab)) ylab else xy$ylab
+        }
+        ## make sure points parameters are right
+
+        n <- length(x)
+        len_color <- length(color)
+        if (len_color > 1) {
+            if (len_color != n) {
+                color <- rep_len(color, n)
+            }
+        } else {
+            if(is.na(color)) color <- "grey60"
+        }
+
+        len_size <- length(size)
+        if (len_size > 1) {
+            if (len_size != n) {
+                size <- rep_len(size, n)
+            }
+        } else {
+            if(is.na(size)) size <- 4
+        }
+
+        len_active <- length(active)
+        if (len_active > 1) {
+            if (len_active != n)
+                stop(paste0("When more than length 1, length of active must match number of points:",
+                            n)
+                )
+        } else {
+            if(is.na(active)) active <- TRUE
+        }
+
+        len_selected <- length(selected)
+        if (len_selected > 1) {
+            if (len_selected != n)
+                stop(paste0("When more than length 1, length of selected must match number of points:",
+                            n)
+                )
+        } else {
+            if(is.na(selected)) selected <- FALSE
+        }
+
+        len_glyph <- length(glyph)
+        if (len_glyph > 1) {
+            if (len_glyph != n)
+                stop(paste0("When more than length 1, length of glyph must match number of points:",
+                            n)
+                )
+        } else {
+            if(is.na(glyph)) glyph <- "ccircle"
+        }
+        # linkingGroup is set after the plot is created
+        linkingGroup <- args[["linkingGroup"]]
+        args$linkingGroup <- NULL
+        # n dimensional states NA check
+        args$x <- x
+        args$y <- y
+        args$color <- color
+        args$glyph <- glyph
+        args$size <- size
+        args$active <- active
+        args$selected <- selected
+
+        if(is.null(by)) {
+            args <- l_na_omit("l_plot.default", args)
+
+            plot <- do.call(
+                loonPlotFactory,
+                c(
+                    args,
+                    list(factory_tclcmd = '::loon::plot',
+                         factory_path = 'plot',
+                         factory_window_title = 'loon scatterplot',
+                         parent = parent,
+                         xlabel = xlabel,
+                         ylabel = ylabel,
+                         title = title,
+                         showLabels = showLabels,
+                         showScales = showScales,
+                         showGuides = showGuides,
+                         guidelines = guidelines,
+                         guidesBackground = guidesBackground,
+                         foreground = foreground,
+                         background = background)
+                )
+            )
+
+            if(!is.null(linkingGroup)) {
+                l_configure(plot,
+                            linkingGroup = linkingGroup,
+                            sync = sync)
+            }
+
+
+            class(plot) <- c("l_plot", class(plot))
+            return(plot)
+
+        } else {
+
+            # convert all types of 'by' to a data frame
+            byDeparse <- deparse(substitute(by))
+
+            if(is.atomic(by)) {
+                if(length(by) == n) {
+                    by <- setNames(data.frame(by, stringsAsFactors = FALSE), byDeparse)
+                } else {
+                    # 'by' is a char vector
+                    # 'x' should be a data.frame
+                    if(!is.data.frame(xOrigin))
+                        stop("If 'by' are variable names, ",
+                             xlab,
+                             " should be a data frame")
+                    by <- xOrigin[by]
+                }
+            } else {
+
+                if(is.null(names(by))) {
+
+                    by <- as.data.frame(by, stringsAsFactors = FALSE)
+                    names(by) <- NULL
+                } else {
+                    by <- as.data.frame(by, stringsAsFactors = FALSE)
+                }
+            }
+
+            plots <- loonFacets(type = "l_plot",
+                                by,
+                                args,
+                                byDeparse = byDeparse,
+                                layout = match.arg(layout),
+                                connectedScales = match.arg(connectedScales),
+                                by_args = Filter(Negate(is.null), by_args),
+                                linkingGroup = linkingGroup,
+                                sync = sync,
+                                parent = parent,
                                 xlabel = xlabel,
                                 ylabel = ylabel,
                                 title = title,
+                                factory_tclcmd = '::loon::plot',
+                                factory_path = 'plot',
+                                factory_window_title = 'loon scatterplot',
                                 showLabels = showLabels,
                                 showScales = showScales,
                                 showGuides = showGuides,
                                 guidelines = guidelines,
                                 guidesBackground = guidesBackground,
                                 foreground = foreground,
-                                background = background,
-                                ...)
+                                background = background)
 
-    } else {
-        ## Get x, y, xlab, ylab
-        ## similar to plot.default use of xy.coords
-        xy <- xy.coords(x, y, xlabel, ylabel)
-        x <- xy$x
-        y <- xy$y
-        if (is.null(xy$xlab)) xy$xlab <- ""
-        if (is.null(xy$ylab)) xy$ylab <- ""
-        ## make sure points parameters are right
-        if (length(color) > 1) {
-            if (length(color) != length(x)) {
-                color <- rep_len(color, length(x))
-            }
+            return(plots)
         }
-        if (!is.numeric(size)) stop("size must be numeric")
-        if (length(size) > 1) {
-            if (length(size) != length(x)) {
-                size <- rep_len(size, length(x))
-            }
-        }
-        if (length(active) > 1) {
-            if (length(active) != length(x))
-                stop(paste0("When more than length 1, length of active must match number of points:",
-                            length(x))
-                )
-        }
-        if (length(selected) > 1) {
-            if (length(selected) != length(x))
-                stop(paste0("When more than length 1, length of selected must match number of points:",
-                            length(x))
-                )
-        }
-        if (length(glyph) > 1) {
-            if (length(glyph) != length(x))
-                stop(paste0("When more than length 1, length of glyph must match number of points:",
-                            length(x))
-                )
-        }
-        plot <- loonPlotFactory('::loon::plot', 'plot', 'loon scatterplot',
-                                parent,
-                                x = x, y = y,
-                                color = color,
-                                glyph = glyph,
-                                size = size,
-                                active = active,
-                                selected = selected,
-                                xlabel = xy$xlab,
-                                ylabel = xy$ylab,
-                                title = title,
-                                showLabels = showLabels,
-                                showScales = showScales,
-                                showGuides = showGuides,
-                                guidelines = guidelines,
-                                guidesBackground = guidesBackground,
-                                foreground = foreground,
-                                background = background,
-                                ...)
-
     }
-
-    class(plot) <- c("l_plot", class(plot))
-    return(plot)
 }
 
-
-
+byArgs <-c("nrow",
+           "ncol",
+           "labelLocation",
+           "labelBackground",
+           "labelForeground",
+           "labelBorderwidth",
+           "labelRelief")
