@@ -1,25 +1,31 @@
 
 #' @rdname loonGrob
-#' 
-#' @examples 
-#' 
+#'
+#' @examples
+#'
 #' \dontrun{
 #' ## navgraph examples
-#' 
+#'
 #' ng <- l_navgraph(oliveAcids, separator='-', color=olive$Area)
-#' 
+#'
+#' # To print directly use either
+#' plot(ng)
+#' # or
+#' grid.loon(ng)
+#' # or to save structure
+#' lgrob <- loonGrob(ng)
 #' library(grid)
-#' navGrob <- loonGrob(ng)
 #' grid.newpage()
-#' grid.draw(navGrob)
+#' grid.draw(lgrob)
 #' }
-#' 
+#'
 #' @export
 loonGrob.l_navgraph <- function(target, name = NULL, gp = NULL, vp = NULL){
-    
+
     widget <- target
     graph <-  widget$graph
     gTree(children = gList(loonGrob(graph, name = "l_navgraph")),
-          name = name, gp = gp, vp = vp)
+          name = if (is.null(name)) "navgraph" else name,
+          gp = gp, vp = vp)
 }
 
