@@ -20,7 +20,7 @@
 	}
 	
 	foreach state {x y tag} {
-	    set ${state}_var [uplevel #0 ${ns}::my varname $state]
+	    set ${state}_var [uplevel #0 [list ${ns}::my varname $state]]
 	}
 	
 	my MakeStateBinding
